@@ -1,5 +1,5 @@
 package Audio::Nama::Assign;
-use Modern::Perl;
+use Modern::Perl '2020';
 our $VERSION = 1.0;
 use 5.008;
 use feature 'state';
@@ -23,8 +23,6 @@ our %EXPORT_TAGS = ( 'all' => [ qw(
 		serialize
 		assign
 		assign_singletons
-		store_vars
-		json_out
 		yaml_in
 		json_in
 		json_out
@@ -53,9 +51,17 @@ use Carp;
 	$mixer_out_format 				$config->{mixer_out_format}
 	$sample_rate					$config->{sample_rate}
 	$ecasound_tcp_port 				$config->{engine_tcp_port}
-	$ecasound_globals				$config->{engine_globals}
+	$ecasound_globals				$config->{ecasound_globals}
 	$ecasound_buffersize 			$config->{engine_buffersize} 
 	$realtime_profile 				$config->{realtime_profile}
+	$jack_seek_delay    			$config->{engine_base_jack_seek_delay}
+	$jack_transport_mode 			$config->{jack_transport_mode}
+	$ecasound_jack_client_name 		$config->{ecasound_jack_client_name}
+	$ecasound_engine_name 			$config->{ecasound_engine_name}
+	$midi_engine_name 				$config->{midi_engine_name}
+	$use_midi 						$config->{use_midi}
+	$midi_record_buffer 			$config->{midi_record_buffer}
+	$midi_default_input_channel 	$config->{midi_default_input_channel}
 	$eq 							$mastering->{fx_eq}
 	$low_pass 						$mastering->{fx_low_pass}
 	$mid_pass						$mastering->{fx_mid_pass}
@@ -68,23 +74,22 @@ use Carp;
 	$press_space_to_start_transport $config->{press_space_to_start}
 	$execute_on_project_load 		$config->{execute_on_project_load}
 	$initial_mode 					$config->{initial_mode}
-	$midish_enable 					$config->{use_midish}
 	$quietly_remove_tracks 			$config->{quietly_remove_tracks}
 	$use_jack_plumbing 				$config->{use_jack_plumbing}
-	$jack_seek_delay    			$config->{engine_base_jack_seek_delay}
-	$use_monitor_version_for_mixdown $config->{sync_mixdown_and_monitor_version_numbers} 
+	$use_playback_version_for_mixdown $config->{sync_mixdown_and_playback_version_numbers} 
 	$mixdown_encodings 				$config->{mixdown_encodings}
 	$volume_control_operator 		$config->{volume_control_operator}
 	$serialize_formats  	        $config->{serialize_formats}
 	$use_git						$config->{use_git}
-	$autosave						$config->{autosave}
-	$beep_command 					$config->{beep_command}
-	$hotkey_beep					$config->{hotkey_beep}
 	$eager							$mode->{eager}
+	$waveform_height 				$config->{waveform_height}
 	$alias							$config->{alias}
 	$hotkeys						$config->{hotkeys}
 	$new_track_rw					$config->{new_track_rw}
-	$hotkeys_always					$config->{hotkeys_always}
+	$playback_jump_seconds		$config->{playback_jump_seconds}
+	$mark_bump_seconds				$config->{mark_bump_seconds}
+	$mark_replay_seconds			$config->{mark_replay_seconds}
+	$seek_end_margin			   $config->{seek_end_margin}
 	$use_pager     					$config->{use_pager}
 	$use_placeholders  				$config->{use_placeholders}
     $edit_playback_end_margin  		$config->{edit_playback_end_margin}
@@ -104,11 +109,18 @@ use Carp;
 	$osc_reply_port 				$config->{osc_reply_port}
 	$remote_control_port 			$config->{remote_control_port}
 	$engines						$config->{engines}
+	$default_waveform_height 		$config->{default_waveform_height}
+	$loop_chain_channel_width 		$config->{loop_chain_channel_width}
+	$waveform_pixels_per_second		$config->{waveform_pixels_per_second}
+	$display_waveform				$config->{display_waveform}
+	$ticks_per_quarter_note 		$config->{ticks_per_quarter_note}
+	$use_metronome					$config->{use_metronome}
+	%beep							$config->{beep}
 
 ) };
 sub var_map {  $var_map } # to allow outside access while keeping
                           # working lexical
-sub config_vars { grep {$_ ne '**' } keys %$var_map }
+sub config_vars { keys %$var_map }
 
 sub assign {
   # Usage: 
@@ -119,7 +131,7 @@ sub assign {
   #	class => $class
   #	);
 
-	logsub("&assign");
+	logsub((caller(0))[3]);
 	
 	my %h = @_; # parameters appear in %h
 	my $class;
@@ -263,6 +275,7 @@ $jack
 $fx
 $fx_cache
 $text
+$term
 $gui
 $midi
 $help
@@ -271,7 +284,7 @@ $project
 
 );
 sub assign_singletons {
-	logsub('&assign_singletons');
+	logsub((caller(0))[3]);
 	my $ref = shift;
 	my $data = $ref->{data} or die "expected data got undefined";
 	my $class = $ref->{class} // 'Audio::Nama';
@@ -325,11 +338,11 @@ sub serialize_and_write {
 			qr/ ^ 			# beginning anchor
 			([\%\@\$]) 		# first character, sigil
 			([\w:]+)		# identifier, possibly perl namespace 
-			(?:->{(\w+)})?  # optional hash key for new hash-singleton vars
+			(?:->\{(\w+)})?  # optional hash key for new hash-singleton vars
 			$ 				# end anchor
 			/x;
 sub serialize {
-	logsub("&serialize");
+	logsub((caller(0))[3]);
 
 	my %h = @_;
 	my @vars = @{ $h{vars} };
@@ -403,7 +416,7 @@ sub serialize {
 }
 
 sub json_out {
-	logsub("&json_out");
+	logsub((caller(0))[3]);
 	my $data_ref = shift;
 	my $type = ref $data_ref;
 	croak "attempting to code wrong data type: $type"
@@ -412,7 +425,7 @@ sub json_out {
 }
 
 sub json_in {
-	logsub("&json_in");
+	logsub((caller(0))[3]);
 	my $json = shift;
 	my $data_ref = decode_json($json);
 	$data_ref
@@ -420,11 +433,15 @@ sub json_in {
 
 sub yaml_in {
 	
-	# logsub("&yaml_in");
+	# logsub((caller(0))[3]);
 	my $input = shift;
 	my $yaml = $input =~ /\n/ # check whether file or text
 		? $input 			# yaml text
-		: read_file($input);	# file name
+		: do
+			{
+				logpkg(__FILE__,__LINE__,'debug',"filename: $input"); 
+				read_file($input);	# file name
+			};
 	if ($yaml =~ /\t/){
 		croak "YAML file: $input contains illegal TAB character.";
 	}
